/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 * @author Abner Jose de Faria Silva <abner.silva@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtk.h>
#include <glib/gi18n.h>

#include "tapiocaui_shell_contact_list.h"

void
tapiocaui_shell_contact_list_contact_authorized_cb (TapiocaContactList *tcl,
                                                    const gchar        *addr,
                                                    gpointer            user_data)
{
    TapiocaUIBuddyData *buddy = NULL;
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    gchar *user_id = g_strdup (addr);

    buddy = tapiocaui_db_find_buddy_by_id (self->priv->db_buddy, TAPIOCAUI_DB_ENTRY_BUDDY, user_id);

    if (buddy)
        buddy->subscription_state = TAPIOCA_CONTACT_LIST_SUBSCRIPTION_STATE_BOTH;

    if (self->priv->presence) {
        gchar *contact[] = { user_id, NULL };
        if (!(tapioca_presence_request_status (self->priv->presence, (const gchar **) contact))) {
            tapiocaui_debug ("failed on request presence for %s", user_id);
        }
    }
    g_free (user_id);
}

void
tapiocaui_shell_contact_list_contact_denied_cb (TapiocaContactList *tcl,
                                                const gchar        *addr,
                                                gpointer            user_data)
{
    TapiocaUIBuddyData *buddy = NULL;
    gchar *user_id = g_strdup (addr);
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);

    GtkWidget *dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (self->priv->win),
                                                            GTK_DIALOG_MODAL,
                                                            GTK_MESSAGE_INFO,
                                                            GTK_BUTTONS_NONE,
                                                            _("The user <b>%s</b> has denied your authorization request"),
                                                            user_id);

    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                            GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);

    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);

    gtk_dialog_run (GTK_DIALOG (dialog));

    buddy = tapiocaui_db_find_buddy_by_id (self->priv->db_buddy, TAPIOCAUI_DB_ENTRY_BUDDY, user_id);

    gtk_widget_destroy (GTK_WIDGET (dialog));

    if (buddy)
        tapiocaui_db_remove_buddy (self->priv->db_buddy, buddy);

    g_free (user_id);
}

void
tapiocaui_shell_contact_list_auth_requested_cb (TapiocaContactList  *tcl,
                                                const gchar *addr,
                                                gpointer user_data)
{
    guint response;
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    gchar *user_id = g_strdup (addr);
    const gchar *groups[] = { "", NULL };

    GtkWidget *dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW (self->priv->win),
                                                            GTK_DIALOG_MODAL,
                                                            GTK_MESSAGE_QUESTION,
                                                            GTK_BUTTONS_NONE,
                                                            _("The user <b>%s</b> has added you to his/her contact list. Authorize?"),
                                                            user_id);

    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                            GTK_STOCK_NO, GTK_RESPONSE_CANCEL,
                            GTK_STOCK_YES, GTK_RESPONSE_OK, NULL);

    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);

    response = gtk_dialog_run (GTK_DIALOG (dialog));

    switch (response) {
        case GTK_RESPONSE_OK:
            {
                TapiocaUIBuddyData *buddy;

                if (!tapioca_contact_list_authorize_contact (self->priv->contact_list,
                                                             user_id, groups))
                    tapiocaui_debug ("Failed to authorize contact");

                buddy = tapiocaui_db_entry_new (TAPIOCAUI_DB_ENTRY_BUDDY);
                buddy->id = g_strdup (user_id);
                buddy->name = g_strdup (user_id);
                buddy->group = g_strdup ("");
                buddy->subscription_state = TAPIOCA_CONTACT_LIST_SUBSCRIPTION_STATE_BOTH;
                tapiocaui_db_insert_buddy (self->priv->db_buddy, buddy);
            }
            break;
        default:
            if (!tapioca_contact_list_deny_contact (self->priv->contact_list,
                                                    user_id))
                tapiocaui_debug ("Failed to deny contact");
            break;
    }

    gtk_widget_destroy (GTK_WIDGET (dialog));

    g_free (user_id);
}

void
tapiocaui_shell_contact_list_contacts_retrieved_cb (TapiocaContactList  *tcl,
                                                    const gchar *addr,
                                                    const gchar *alias,
                                                    const gchar *group,
                                                    guint state,
                                                    gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    TapiocaUIBuddyData *buddy = NULL;

    /* Return if empty string on address
       See WORKAROUND to set end of contact list 
    */
    if (!g_ascii_strcasecmp (addr, ""))
	return;

    buddy = tapiocaui_db_entry_new (TAPIOCAUI_DB_ENTRY_BUDDY);
    buddy->id = g_strdup (addr);
    buddy->name = g_strdup (alias);
    buddy->group = g_strdup (group);
    buddy->subscription_state = state;

    /* Syncronize remote buddy with local buddys */
    tapiocaui_db_sync_buddy (self->priv->db_buddy, buddy);

    /* Request presence for buddy */
    if (self->priv->presence) {
        gchar *contact[] = { buddy->id, NULL };
        if (!(tapioca_presence_request_status (self->priv->presence, (const gchar **) contact))) {
            tapiocaui_debug ("failed on request presence for %s", buddy->id);
        }
    }
    tapiocaui_db_commit (self->priv->db_buddy);
}

void
tapiocaui_shell_contact_list_contact_info_retrieved_cb (TapiocaContactList *tcl,
                                                        const gchar *addr,
                                                        gpointer *info,
                                                        gpointer user_data)
{
    tapiocaui_debug ("Not implemented");
}
