/*
 * Tapiocaui app
 * Copyright (C) 2005 INdT
 * @author Renato Araujo Oliveira Filho <renato.filho@indt.org.br>
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 * @author Abner Jose de Faria Silva <abner.silva@indt.org.br>
 * @author Marcio Macedo <marcio.macedo@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA02111-1307USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtk.h>

#include "tapiocaui_shell_mainwin.h"
#include "tapiocaui_shell_session.h"
#include "tapiocaui_shell_presence.h"
#include "tapiocaui_shell_contact_list.h"
#ifdef ENABLE_SYSTRAY
#include "tapiocaui_systray.h"
#endif
#define TAPIOCA_PROTOCOL "xmpp"

gboolean
tapiocaui_shell_mainwin_delete_cb (GtkWidget *widget,
                                   GdkEvent *event,
                                   gpointer user_data)
{
#ifdef ENABLE_SYSTRAY
    gtk_widget_hide(widget); 
#else
    gtk_main_quit ();
#endif
    return TRUE;
}

void
tapiocaui_shell_mainwin_login_cb (TapiocaUIMainWin *win,
                                  const gchar *user,
                                  const gchar *proxy_password,
                                  gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    GHashTable *table = NULL;
    gboolean reusing_connection = FALSE;

    if (!self->priv->tc) {
        self->priv->tc = tapioca_connection_manager_create_connection (
                self->priv->tcm, TAPIOCA_PROTOCOL);
        if (self->priv->tc == NULL) {
            tapiocaui_main_win_cancel_login (win, -1);
            return;
        }
    }
    else {
        reusing_connection = TRUE;
    }

    table = g_hash_table_new_full (g_str_hash, g_str_equal,
            NULL, g_free);

    g_hash_table_insert (table, "username", g_strdup (user));
    g_hash_table_insert (table, "password", g_strdup (proxy_password));
    /* g_hash_table_insert (table, "server-address", g_strdup ("amessage.info")); */
    g_hash_table_insert (table, "server-address", g_strdup ("talk.google.com"));
    /* g_hash_table_insert (table, "server-port",  g_strdup ("443")); */
    g_hash_table_insert (table, "server-port",  g_strdup ("5222"));
    g_hash_table_insert (table, "use-encryption", g_strdup ("1"));
    /* g_hash_table_insert (table, "proxy-type", g_strdup_printf ("%d", TAPIOCA_PROXY_TYPE_HTTPS)); */
    /* g_hash_table_insert (table, "proxy-address",  g_strdup ("proxy")); */
    /* g_hash_table_insert (table, "proxy-port",  g_strdup ("8080")); */

    g_signal_connect (G_OBJECT (self->priv->tc),
                      "incoming-session",
                      G_CALLBACK (tapiocaui_shell_incoming_session_cb),
                      self);
    g_signal_connect (G_OBJECT (self->priv->tc),
                      "state-changed",
                      G_CALLBACK (tapiocaui_shell_state_changed_cb),
                      self);
    g_signal_connect (G_OBJECT (self->priv->tc),
                      "error",
                      G_CALLBACK (tapiocaui_shell_connection_error_cb),
                      self);

    if (!(tapioca_connection_connect (self->priv->tc, user, table))) {
        tapiocaui_debug ("error on tapioca_connect");
        tapiocaui_main_win_cancel_login (win, -1);
        g_hash_table_destroy (table);
        g_object_unref (G_OBJECT (self->priv->tc));
        self->priv->tc = NULL;
        return;
    }

    g_hash_table_destroy (table);

    if (reusing_connection)
       return;

    /* Check if remote contact list is supported */
    if (!(self->priv->contact_list = TAPIOCA_CONTACT_LIST (tapioca_connection_get_contact_list (self->priv->tc)))) {
        tapiocaui_debug ("Connection does not support remote contact list");
    }
    else {
        /* Connect all contact list signals */
        g_signal_connect (G_OBJECT (self->priv->contact_list),
                          "contact-authorized",
                          G_CALLBACK (tapiocaui_shell_contact_list_contact_authorized_cb),
                          self);
        g_signal_connect (G_OBJECT (self->priv->contact_list),
                          "contact-denied",
                          G_CALLBACK (tapiocaui_shell_contact_list_contact_denied_cb),
                          self);
        g_signal_connect (G_OBJECT (self->priv->contact_list),
                          "authorization-requested",
                          G_CALLBACK (tapiocaui_shell_contact_list_auth_requested_cb),
                          self);
        g_signal_connect (G_OBJECT (self->priv->contact_list),
                          "contacts-retrieved",
                          G_CALLBACK (tapiocaui_shell_contact_list_contacts_retrieved_cb),
                          self);
        g_signal_connect (G_OBJECT (self->priv->contact_list),
                          "contact-info-retrieved",
                          G_CALLBACK (tapiocaui_shell_contact_list_contact_info_retrieved_cb),
                          self);
    }

    /* Check if presence is supported */
    if (!(self->priv->presence = TAPIOCA_PRESENCE (tapioca_connection_get_presence (self->priv->tc)))) {
        tapiocaui_debug ("Connection does not support presence");
    }
    else {
        /* Connect all window signals for presence */
        g_signal_connect (G_OBJECT (self->priv->win),
                          "buddy-added",
                          G_CALLBACK (tapiocaui_shell_buddy_added_cb),
                          self);
        g_signal_connect (G_OBJECT (self->priv->win),
                          "buddy-removed",
                          G_CALLBACK (tapiocaui_shell_buddy_removed_cb),
                          self);
        g_signal_connect (G_OBJECT (self->priv->win),
                          "buddy-updated",
                          G_CALLBACK (tapiocaui_shell_buddy_updated_cb),
                          self);
        g_signal_connect (G_OBJECT (self->priv->win),
                          "account-status-updated",
                          G_CALLBACK (tapiocaui_shell_account_status_updated_cb),
                          self);

        /* Connect all presence signals */
        g_signal_connect (G_OBJECT (self->priv->presence),
                          "status-updated",
                          G_CALLBACK (tapiocaui_shell_presence_status_up_cb),
                          self);
        g_signal_connect (G_OBJECT (self->priv->presence),
                          "registered",
                          G_CALLBACK (tapiocaui_shell_presence_registered_cb),
                          self);
        g_signal_connect (G_OBJECT (self->priv->presence),
                          "unregistered",
                          G_CALLBACK (tapiocaui_shell_presence_unregistered_cb),
                          self);
    }
}

void
tapiocaui_shell_mainwin_login_cancelled_cb (TapiocaUIMainWin *win,
                                            gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    g_object_unref (self->priv->tc);
    self->priv->tc = NULL;
}

void
tapiocaui_shell_mainwin_logout_cb (TapiocaUIMainWin *win,
                                   gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);

    /* Disconnect all window signals for presence */
    g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->win),
                                          G_CALLBACK (tapiocaui_shell_buddy_added_cb),
                                          self);
    g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->win),
                                          G_CALLBACK (tapiocaui_shell_buddy_removed_cb),
                                          self);
    g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->win),
                                          G_CALLBACK (tapiocaui_shell_buddy_updated_cb),
                                          self);
    g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->win),
                                          G_CALLBACK (tapiocaui_shell_account_status_updated_cb),
                                          self);

    /* Disconnect all tapioca connection signals */
    g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->tc),
                                          G_CALLBACK (tapiocaui_shell_incoming_session_cb),
                                          self);
    g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->tc),
                                          G_CALLBACK (tapiocaui_shell_state_changed_cb),
                                          self);
    g_signal_handlers_disconnect_by_func (G_OBJECT (self->priv->tc),
                                          G_CALLBACK (tapiocaui_shell_connection_error_cb),
                                          self);

    tapioca_connection_disconnect (self->priv->tc);

    g_object_unref (self->priv->db_buddy);
    tapiocaui_main_win_logout (TAPIOCAUI_MAIN_WIN (self->priv->win));
#ifdef ENABLE_SYSTRAY    
    tapiocaui_systray_set_icon_offline();
#endif    
}

void
tapiocaui_shell_mainwin_call_cb (TapiocaUIMainWin *win,
                                 const gchar *user,
                                 gpointer user_data)
{
    TapiocaUIShellActiveComm *active_comm;
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);

    tapiocaui_debug ("trying to make a call");
    active_comm = tapiocaui_shell_get_communication (self, user);
    tapiocaui_shell_call_buddy (self, active_comm);
}


void
tapiocaui_shell_mainwin_open_chat_cb (TapiocaUIMainWin *win,
                                      const gchar *user,
                                      gpointer user_data)
{
    TapiocaUIShellActiveComm *comm;
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    TapiocaUIBuddyData *buddy = NULL;

    tapiocaui_debug ("");

    buddy = tapiocaui_db_find_buddy_by_id (self->priv->db_buddy,
                                           TAPIOCAUI_DB_ENTRY_BUDDY,
                                           user);

    comm = tapiocaui_shell_get_communication (self, user);
}

void
tapiocaui_shell_buddy_added_cb (TapiocaUIMainWin *win,
                                const gchar *user,
                                gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    gchar *user_id = g_strdup (user);

    if (self->priv->contact_list) {
        TapiocaUIBuddyData *buddy = tapiocaui_db_find_buddy_by_id (self->priv->db_buddy,
                                                                   TAPIOCAUI_DB_ENTRY_BUDDY,
                                                                   user_id);
        if (buddy) {
            const gchar *groups[] = { buddy->group, NULL };
            if (!tapioca_contact_list_add_contact (self->priv->contact_list,
                                                   buddy->id,
                                                   buddy->name,
                                                   groups))
                tapiocaui_debug ("failed on add contact");

            buddy->subscription_state = TAPIOCA_CONTACT_LIST_SUBSCRIPTION_STATE_NONE;
        }
    }

    if (self->priv->presence) {
        gchar *contact[] = { user_id, NULL };
        if (!(tapioca_presence_request_status (self->priv->presence, (const gchar **) contact))) {
            tapiocaui_debug ("failed on request presence for %s", user_id);
        }
    }
    g_free (user_id);
}

void
tapiocaui_shell_buddy_removed_cb (TapiocaUIMainWin *win,
                                  const gchar *user,
                                  gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);
    gchar * user_id = g_strdup (user);

    if (self->priv->contact_list) {
        if (!tapioca_contact_list_remove_contact (self->priv->contact_list, user_id))
            tapiocaui_debug ("Failed on remove contact from remote contact list");
    }

    if (self->priv->presence) {
        gchar *contact[] = { user_id, NULL };
        if (!(tapioca_presence_release_status (self->priv->presence, (const gchar **) contact))) {
            tapiocaui_debug ("failed on release presence for %s", user_id);
        }
    }
    g_free (user_id);
}

void
tapiocaui_shell_buddy_updated_cb (TapiocaUIMainWin *win,
                                  const gchar *old_user,
                                  const gchar *new_user,
                                  gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);

    if (self->priv->presence) {
        gchar *contact[] = { g_strdup (old_user), NULL };
        if (!(tapioca_presence_release_status (self->priv->presence, (const gchar **) contact))) {
            tapiocaui_debug ("failed on release presence for %s", old_user);
        }

        g_free (contact[0]);
        contact[0] = g_strdup (new_user);

        if (!(tapioca_presence_request_status (self->priv->presence, (const gchar **) contact))) {
            tapiocaui_debug ("failed on request presence for %s", new_user);
        }
        g_free (contact[0]);
    }
}

void
tapiocaui_shell_account_status_updated_cb (TapiocaUIMainWin *win,
                                           guint status,
                                           const gchar *status_desc,
                                           gpointer user_data)
{
    TapiocaUIShell *self = TAPIOCAUI_SHELL (user_data);

    if (self->priv->presence) {
        if (!(tapioca_presence_set_status (self->priv->presence, status, status_desc))) {
            tapiocaui_debug ("failed setting presence status");
            return;
        }
    }
}
